#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
#import shutil
import string
import pwd
#import subprocess
#from subprocess import Popen
from pwd import getpwnam
#Cambio a PyQt5
#from PyQt4 import QtGui, QtCore
from PyQt5 import QtWidgets, QtGui, QtCore
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QMessageBox

from ui_user import Ui_MainWindow
from cgauser import *
from cgauserconfig import *


class CgaPasswordError(Exception): pass 
class CgaEmptyFields(Exception): pass
class CgaInvalidCharacter(Exception): pass


VALID_CHARS = string.ascii_letters + "0123456789"
VALID_CHARS_LOWER = string.ascii_lowercase + "0123456789"
VALID_CHARS_EXTENDED = VALID_CHARS + "ñ" + "áéíóú" + "ç" + "-_." + " "


#class SUser(QtGui.QMainWindow, Ui_MainWindow):
class SUser(QtWidgets.QMainWindow, Ui_MainWindow):
	
	def __init__(self, parent = None):
		super(SUser, self).__init__(parent)
		self.setupUi(self)
		self.settings = QtCore.QSettings()
		self.signals()
		self.center()


	def signals(self):
#		QtCore.QObject.connect(self.b_aceptar, QtCore.SIGNAL("clicked()"), self.create_user)
		self.b_aceptar.clicked.connect(self.create_user)
#		QtCore.QObject.connect(self.b_cancelar, QtCore.SIGNAL("clicked()"), self.close)
		self.b_cancelar.clicked.connect(self.close)
#		QtCore.QObject.connect(self.checkPass, QtCore.SIGNAL("stateChanged(int)"), self.passVisible)
		self.checkPass.stateChanged.connect(self.passVisible)
	def create_user(self):
		#usuario = unicode(self.texto_usuario.text().toUtf8(), encoding="UTF-8")
		#login = unicode(self.texto_id.text().toUtf8(), encoding="UTF-8") 
		#pwd1 = unicode(self.texto_pwd1.text().toUtf8(), encoding="UTF-8") 
		#pwd2 = unicode(self.texto_pwd2.text().toUtf8(), encoding="UTF-8") 
		#mensaje = None
		usuario = str(self.texto_usuario.text())
		login = str(self.texto_id.text()) 
		pwd2 = str(self.texto_pwd2.text()) 
		pwd1 = str(self.texto_pwd1.text()) 
		mensaje = None

		try:
			self._data_ok(usuario, login, pwd1, pwd2)
			self.is_sane(usuario)
			self.is_sane(pwd1)
			self.is_sane(pwd2)
			self.is_sane(login, restricted = True)
			
		except CgaInvalidCharacter as e:
			mensaje = str(""" Uno de los campos contiene caracteres no válidos. Los caracteres válidos para cada campo son:
						\n*Nombre y Contraseña:
						'%s'
						\n*Login:
						'%s' """ % (VALID_CHARS_EXTENDED, VALID_CHARS_LOWER))
			#mensaje = mensaje.decode('utf8')
			# mensaje = mensaje.encode('utf8')

		except CgaEmptyFields:
			mensaje = u"Todos los campos son obligatorios.\nPor favor compruebe que ha rellenado todos los campos."
		except CgaPasswordError:
			mensaje = u"Las contraseñas son distintas.\nPor favor asegúrese de que escribe la contraseña correctamente en los dos campos."
		
		if mensaje is not None:
			self.msgbox(mensaje, "critical")
			return

		error = None
		try:
			CgaUser(login, usuario, pwd1).save()

#		except CgaUserExists, msg:
#			print "El usuario ya existe"
#			error = msg
#		except CgaUserCreationError, msg:
#			print "Ha ocurrido un error al crear el usuario"
#			error = msg
		except CgaUserExists as msg:
			print ("El usuario ya existe")
			error = msg
		except CgaUserCreationError as msg:
			print ("Ha ocurrido un error al crear el usuario")
			error = msg

		if error:
			msg =  u"No se pudo crear el usuario '%s': \n %s" % (login, error)
			self.msgbox(msg, "critical")
		else:
			msg = u"Usuario local '%s' creado correctamente." % login
			self.msgbox(msg, "information")
			QtWidgets.qApp.closeAllWindows()



	def msgbox(self, msg, tipo):
		if tipo == "information":
			QtWidgets.QMessageBox.information(self, u"Información", str(msg))
		elif tipo == "critical":
			QtWidgets.QMessageBox.critical(self, u"Error", str(msg), QMessageBox.Ok)


	def _data_ok(self, usuario, login, pwd1, pwd2):
		if usuario == "" or login == "" or pwd1 == "" or pwd2 == "":
			raise CgaEmptyFields
		if pwd1 != pwd2:
			raise CgaPasswordError


	def is_sane(self, cad, restricted=False):
		valid = (VALID_CHARS_EXTENDED, VALID_CHARS_LOWER)[restricted]
		res = map(lambda k: k in list(valid), cad)
		if not all(res):
			raise CgaInvalidCharacter(cad)


	def passVisible(self):
		if self.checkPass.isChecked():
			self.texto_pwd1.setEchoMode(QtWidgets.QLineEdit.Normal)
			self.texto_pwd2.setEchoMode(QtWidgets.QLineEdit.Normal)
		else:
			self.texto_pwd1.setEchoMode(QtWidgets.QLineEdit.Password)
			self.texto_pwd2.setEchoMode(QtWidgets.QLineEdit.Password)


	def center(self):
#		screen = QtGui.QDesktopWidget().screenGeometry()
		screen = QtWidgets.QDesktopWidget().screenGeometry()
		size = self.geometry()
		self.move(int((screen.width() - size.width())/2), int((screen.height() - size.height())/2))


	def close(self):
#		QtGui.qApp.closeAllWindows()
		QtWidgets.qApp.closeAllWindows()



if __name__ == '__main__':
#	app = QtGui.QApplication(sys.argv)
	app = QtWidgets.QApplication(sys.argv)
	c = SUser()
	c.show()
	app.exec_()
		
